@@ -90,9 +90,11 @@ struct connman_service {
 	connman_bool_t login_required;
 	struct connman_ipconfig *ipconfig;
 	struct connman_network *network;
+	struct connman_provider *provider;
 	char **nameservers;
 	char *nameserver;
 	char **domains;
+	char *domainname;
 	/* 802.1x settings from the config files */
 	char *eap;
 	char *identity;
@@ -340,6 +342,8 @@ static void update_nameservers(struct connman_service *service)
 						service->nameservers[i]);
 	} else if (service->nameserver != NULL)
 		connman_resolver_append(ifname, NULL, service->nameserver);
+
+	connman_resolver_flush();
 }
 
 void __connman_service_append_nameserver(struct connman_service *service,
@@ -789,6 +793,24 @@ static void append_ipv4(DBusMessageIter *iter, void *user_data)
 		__connman_ipconfig_append_ipv4(service->ipconfig, iter);
 }
 
+static void append_ipv6(DBusMessageIter *iter, void *user_data)
+{
+	struct connman_service *service = user_data;
+	struct connman_ipconfig *ipv6config;
+
+	if (is_connected(service) == FALSE)
+		return;
+
+	if (service->ipconfig == NULL)
+		return;
+
+	ipv6config = connman_ipconfig_get_ipv6config(service->ipconfig);
+	if (ipv6config == NULL)
+		return;
+
+	__connman_ipconfig_append_ipv6(ipv6config, iter);
+}
+
 static void append_ipv4config(DBusMessageIter *iter, void *user_data)
 {
 	struct connman_service *service = user_data;
@@ -797,6 +819,21 @@ static void append_ipv4config(DBusMessageIter *iter, void *user_data)
 		__connman_ipconfig_append_ipv4config(service->ipconfig, iter);
 }
 
+static void append_ipv6config(DBusMessageIter *iter, void *user_data)
+{
+	struct connman_service *service = user_data;
+	struct connman_ipconfig *ipv6config;
+
+	if (service->ipconfig == NULL)
+		return;
+
+	ipv6config = connman_ipconfig_get_ipv6config(service->ipconfig);
+	if (ipv6config == NULL)
+		return;
+
+	__connman_ipconfig_append_ipv6config(ipv6config, iter);
+}
+
 static void append_dns(DBusMessageIter *iter, void *user_data)
 {
 	struct connman_service *service = user_data;
@@ -838,8 +875,14 @@ static void append_domain(DBusMessageIter *iter, void *user_data)
 {
 	struct connman_service *service = user_data;
 
-	if (is_connected(service) == FALSE)
+	if (is_connected(service) == FALSE && is_connecting(service) == FALSE)
 		return;
+
+	if (service->domainname == NULL)
+		return;
+
+	dbus_message_iter_append_basic(iter,
+				DBUS_TYPE_STRING, &service->domainname);
 }
 
 static void append_domainconfig(DBusMessageIter *iter, void *user_data)
@@ -866,11 +909,29 @@ static void append_proxy(DBusMessageIter *iter, void *user_data)
 		__connman_ipconfig_append_proxy(service->ipconfig, iter);
 }
 
+static void append_provider(DBusMessageIter *iter, void *user_data)
+{
+	struct connman_service *service = user_data;
+
+	DBG("%p %p", service, service->provider);
+
+	if (is_connected(service) == FALSE)
+		return;
+
+	if (service->provider != NULL)
+		__connman_provider_append_properties(service->provider, iter);
+}
+
+
 static void settings_changed(struct connman_service *service)
 {
 	connman_dbus_property_changed_dict(service->path,
 					CONNMAN_SERVICE_INTERFACE, "IPv4",
 							append_ipv4, service);
+
+	connman_dbus_property_changed_dict(service->path,
+					CONNMAN_SERVICE_INTERFACE, "IPv6",
+							append_ipv6, service);
 }
 
 static void ipv4_configuration_changed(struct connman_service *service)
@@ -882,11 +943,17 @@ static void ipv4_configuration_changed(struct connman_service *service)
 							service);
 }
 
+static void ipv6_configuration_changed(struct connman_service *service)
+{
+	connman_dbus_property_changed_dict(service->path,
+					CONNMAN_SERVICE_INTERFACE,
+							"IPv6.Configuration",
+							append_ipv6config,
+							service);
+}
+
 static void dns_changed(struct connman_service *service)
 {
-	if (is_connected(service) == FALSE)
-		return;
-
 	connman_dbus_property_changed_array(service->path,
 				CONNMAN_SERVICE_INTERFACE, "Nameservers",
 					DBUS_TYPE_STRING, append_dns, service);
@@ -902,6 +969,13 @@ static void dns_configuration_changed(struct connman_service *service)
 	dns_changed(service);
 }
 
+static void domain_changed(struct connman_service *service)
+{
+	connman_dbus_property_changed_array(service->path,
+				CONNMAN_SERVICE_INTERFACE, "Domains",
+				DBUS_TYPE_STRING, append_domain, service);
+}
+
 static void domain_configuration_changed(struct connman_service *service)
 {
 	connman_dbus_property_changed_array(service->path,
@@ -910,6 +984,13 @@ static void domain_configuration_changed(struct connman_service *service)
 				DBUS_TYPE_STRING, append_domainconfig, service);
 }
 
+static void proxy_changed(struct connman_service *service)
+{
+	connman_dbus_property_changed_dict(service->path,
+					CONNMAN_SERVICE_INTERFACE, "Proxy",
+							append_proxy, service);
+}
+
 static void append_properties(DBusMessageIter *dict, dbus_bool_t limited,
 					struct connman_service *service)
 {
@@ -1043,6 +1124,11 @@ static void append_properties(DBusMessageIter *dict, dbus_bool_t limited,
 	connman_dbus_dict_append_dict(dict, "IPv4.Configuration",
 						append_ipv4config, service);
 
+	connman_dbus_dict_append_dict(dict, "IPv6", append_ipv6, service);
+
+	connman_dbus_dict_append_dict(dict, "IPv6.Configuration",
+						append_ipv6config, service);
+
 	connman_dbus_dict_append_array(dict, "Nameservers",
 				DBUS_TYPE_STRING, append_dns, service);
 
@@ -1056,6 +1142,8 @@ static void append_properties(DBusMessageIter *dict, dbus_bool_t limited,
 				DBUS_TYPE_STRING, append_domainconfig, service);
 
 	connman_dbus_dict_append_dict(dict, "Proxy", append_proxy, service);
+
+	connman_dbus_dict_append_dict(dict, "Provider", append_provider, service);
 }
 
 static void append_struct(gpointer value, gpointer user_data)
@@ -1084,6 +1172,58 @@ void __connman_service_list_struct(DBusMessageIter *iter)
 	g_sequence_foreach(service_list, append_struct, iter);
 }
 
+int __connman_service_get_index(struct connman_service *service)
+{
+	if (service == NULL)
+		return -1;
+
+	if (service->network == NULL)
+		return -1;
+
+	return connman_network_get_index(service->network);
+}
+
+void __connman_service_set_domainname(struct connman_service *service,
+						const char *domainname)
+{
+	if (service == NULL)
+		return;
+
+	g_free(service->domainname);
+	service->domainname = g_strdup(domainname);
+
+	domain_changed(service);
+}
+
+const char *__connman_service_get_domainname(struct connman_service *service)
+{
+	if (service == NULL)
+		return NULL;
+
+	return service->domainname;
+}
+
+const char *__connman_service_get_nameserver(struct connman_service *service)
+{
+	if (service == NULL)
+		return NULL;
+
+	return service->nameserver;
+}
+
+void __connman_service_set_proxy_autoconfig(struct connman_service *service,
+                                                        const char *url)
+{
+	if (service == NULL)
+		return;
+
+	if (__connman_ipconfig_set_proxy_autoconfig(service->ipconfig,
+								url) < 0)
+		return;
+
+	proxy_changed(service);
+}
+
 static DBusMessage *get_properties(DBusConnection *conn,
 					DBusMessage *msg, void *user_data)
 {
@@ -1329,29 +1469,51 @@ static DBusMessage *set_property(DBusConnection *conn,
 		domain_configuration_changed(service);
 
 		__connman_storage_save_service(service);
-	} else if (g_str_equal(name, "IPv4.Configuration") == TRUE) {
-		int err;
+	} else if (g_str_equal(name, "IPv4.Configuration") == TRUE ||
+			g_str_equal(name, "IPv6.Configuration")) {
 
+		enum connman_ipconfig_type type = CONNMAN_IPCONFIG_TYPE_UNKNOWN;
+		int err = 0;
+		struct connman_ipconfig *ipv6config;
+
+		DBG("%s", name);
+
+		ipv6config = connman_ipconfig_get_ipv6config(
+						service->ipconfig);
 		if (service->ipconfig == NULL)
 			return __connman_error_invalid_property(msg);
 
 		if (is_connecting(service) ||
-				is_connected(service))
+				is_connected(service)) {
 			__connman_network_clear_ipconfig(service->network,
 							service->ipconfig);
+			__connman_network_clear_ipconfig(service->network,
+								ipv6config);
+		}
+
+		if (g_str_equal(name, "IPv4.Configuration") == TRUE) {
+			type = CONNMAN_IPCONFIG_TYPE_IPV4;
+			err = __connman_ipconfig_set_config(
+					service->ipconfig, type, &value);
+		} else if (g_str_equal(name, "IPv6.Configuration") == TRUE) {
+			type = CONNMAN_IPCONFIG_TYPE_IPV6;
+			err = __connman_ipconfig_set_config(
+						ipv6config, type, &value);
+		}
 
-		err = __connman_ipconfig_set_ipv4config(service->ipconfig,
-								&value);
 		if (err < 0) {
 			if (is_connected(service) ||
 					is_connecting(service))
-				__connman_network_set_ipconfig(service->network,
+				__connman_network_set_ipconfig(
+							service->network,
 							service->ipconfig);
-
 			return __connman_error_failed(msg, -err);
 		}
 
-		ipv4_configuration_changed(service);
+		if (type == CONNMAN_IPCONFIG_TYPE_IPV4)
+			ipv4_configuration_changed(service);
+		else if (type == CONNMAN_IPCONFIG_TYPE_IPV6)
+			ipv6_configuration_changed(service);
 
 		if (is_connecting(service) ||
 				is_connected(service))
@@ -1572,6 +1734,8 @@ __connman_service_connect_type(enum connman_service_type type)
 	 * the first available one if we have no type.
 	 */
 	iter = g_sequence_get_begin_iter(service_list);
+	if (g_sequence_iter_is_end(iter))
+		return NULL;
 	service = g_sequence_get(iter);
 
 	/*
@@ -1859,6 +2023,7 @@ static void service_free(gpointer user_data)
 	g_strfreev(service->domains);
 
 	g_free(service->nameserver);
+	g_free(service->domainname);
 	g_free(service->mcc);
 	g_free(service->mnc);
 	g_free(service->apn);
@@ -1931,6 +2096,8 @@ static void __connman_service_initialize(struct connman_service *service)
 	service->stats.valid = FALSE;
 	service->stats.enabled = FALSE;
 	service->stats.timer = g_timer_new();
+
+	service->provider = NULL;
 }
 
 /**
@@ -2228,6 +2395,8 @@ int __connman_service_indicate_state(struct connman_service *service,
 		update_nameservers(service);
 		dns_changed(service);
 
+		__connman_wpad_start(service);
+
 		__connman_notifier_connect(service->type);
 
 		default_changed();
@@ -2236,6 +2405,8 @@ int __connman_service_indicate_state(struct connman_service *service,
 
 		default_changed();
 
+		__connman_wpad_stop(service);
+
 		update_nameservers(service);
 		dns_changed(service);
 
@@ -2392,11 +2563,11 @@ int __connman_service_connect(struct connman_service *service)
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
 	case CONNMAN_SERVICE_TYPE_SYSTEM:
 	case CONNMAN_SERVICE_TYPE_GPS:
-	case CONNMAN_SERVICE_TYPE_VPN:
 		return -EINVAL;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIMAX:
 	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		break;
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
 		if (service->apn == NULL)
@@ -2440,7 +2611,10 @@ int __connman_service_connect(struct connman_service *service)
 		__connman_ipconfig_enable(service->ipconfig);
 
 		err = __connman_network_connect(service->network);
-	} else
+	} else if (service->type == CONNMAN_SERVICE_TYPE_VPN &&
+					service->provider != NULL)
+		err = __connman_provider_connect(service->provider);
+	else
 		return -EOPNOTSUPP;
 
 	if (err < 0) {
@@ -2460,17 +2634,25 @@ int __connman_service_connect(struct connman_service *service)
 
 int __connman_service_disconnect(struct connman_service *service)
 {
+	struct connman_ipconfig *ipv6config;
 	int err;
 
 	DBG("service %p", service);
 
 	if (service->network != NULL) {
 		err = __connman_network_disconnect(service->network);
-	} else
+	} else if (service->type == CONNMAN_SERVICE_TYPE_VPN &&
+					service->provider != NULL)
+		err = __connman_provider_disconnect(service->provider);
+	else
 		return -EOPNOTSUPP;
 
 	__connman_ipconfig_clear_address(service->ipconfig);
 
+	ipv6config = connman_ipconfig_get_ipv6config(service->ipconfig);
+
+	__connman_ipconfig_clear_address(ipv6config);
+
 	__connman_ipconfig_disable(service->ipconfig);
 
 	if (err < 0) {
@@ -2879,6 +3061,7 @@ static void setup_ipconfig(struct connman_service *service, int index)
 void __connman_service_create_ipconfig(struct connman_service *service,
 								int index)
 {
+	struct connman_ipconfig *ipv6config;
 	const char *ident = service->profile;
 	GKeyFile *keyfile;
 
@@ -2894,6 +3077,12 @@ void __connman_service_create_ipconfig(struct connman_service *service,
 	if (keyfile == NULL)
 		return;
 
+
+	ipv6config = connman_ipconfig_get_ipv6config(service->ipconfig);
+	if (ipv6config != NULL)
+		__connman_ipconfig_load(ipv6config, keyfile,
+					service->identifier, "IPv6.");
+
 	__connman_ipconfig_load(service->ipconfig, keyfile,
 					service->identifier, "IPv4.");
 	g_key_file_free(keyfile);
@@ -3111,7 +3300,7 @@ static void update_from_network(struct connman_service *service,
  *
  * Look up service by network and if not found, create one
  */
-struct connman_service *__connman_service_create_from_network(struct connman_network *network)
+struct connman_service * __connman_service_create_from_network(struct connman_network *network)
 {
 	struct connman_service *service;
 	const char *ident, *group;
@@ -3252,6 +3441,68 @@ void __connman_service_remove_from_network(struct connman_network *network)
 	__connman_service_put(service);
 }
 
+/**
+ * __connman_service_create_from_provider:
+ * @provider: provider structure
+ *
+ * Look up service by provider and if not found, create one
+ */
+struct connman_service *
+__connman_service_create_from_provider(struct connman_provider *provider)
+{
+	struct connman_service *service;
+	const char *ident, *str;
+	char *name;
+	int index = connman_provider_get_index(provider);
+
+	DBG("provider %p", provider);
+
+	ident = __connman_provider_get_ident(provider);
+	if (ident == NULL)
+		return NULL;
+
+	name = g_strdup_printf("vpn_%s", ident);
+	service = __connman_service_get(name);
+	g_free(name);
+
+	if (service == NULL)
+		return NULL;
+
+	service->type = CONNMAN_SERVICE_TYPE_VPN;
+	service->provider = provider;
+	service->autoconnect = FALSE;
+
+	service->state = CONNMAN_SERVICE_STATE_IDLE;
+
+	str = connman_provider_get_string(provider, "Name");
+	if (str != NULL) {
+		g_free(service->name);
+		service->name = g_strdup(str);
+		service->hidden = FALSE;
+	} else {
+		g_free(service->name);
+		service->name = NULL;
+		service->hidden = TRUE;
+	}
+
+	service->strength = 0;
+
+	service->ipconfig = connman_ipconfig_create(index);
+	if (service->ipconfig == NULL)
+		return service;
+
+	connman_ipconfig_set_method(service->ipconfig,
+					CONNMAN_IPCONFIG_METHOD_MANUAL);
+
+	connman_ipconfig_set_data(service->ipconfig, service);
+
+	connman_ipconfig_set_ops(service->ipconfig, &service_ops);
+
+	service_register(service);
+
+	return service;
+}
+
 void __connman_service_stats_update(struct connman_service *service,
 				unsigned int rx_packets, unsigned int tx_packets,
 				unsigned int rx_bytes, unsigned int tx_bytes,
@@ -3439,9 +3690,18 @@ static int service_load(struct connman_service *service)
 		service->passphrase = str;
 	}
 
-	if (service->ipconfig != NULL)
+	if (service->ipconfig != NULL) {
+		struct connman_ipconfig *ipv6config;
+
+		ipv6config = connman_ipconfig_get_ipv6config(
+						service->ipconfig);
+		if (ipv6config != NULL)
+			__connman_ipconfig_load(ipv6config, keyfile,
+					service->identifier, "IPv6.");
+
 		__connman_ipconfig_load(service->ipconfig, keyfile,
 					service->identifier, "IPv4.");
+	}
 
 	service->nameservers = g_key_file_get_string_list(keyfile,
 			service->identifier, "Nameservers", &length, NULL);
@@ -3587,9 +3847,17 @@ update:
 		g_key_file_remove_key(keyfile, service->identifier,
 							"Passphrase", NULL);
 
-	if (service->ipconfig != NULL)
+	if (service->ipconfig != NULL) {
+		struct connman_ipconfig *ipv6config;
+
+		ipv6config = connman_ipconfig_get_ipv6config(service->ipconfig);
+		if (ipv6config != NULL)
+			__connman_ipconfig_save(ipv6config, keyfile,
+						service->identifier, "IPv6.");
+
 		__connman_ipconfig_save(service->ipconfig, keyfile,
 					service->identifier, "IPv4.");
+	}
 
 	if (service->nameservers != NULL) {
 		guint len = g_strv_length(service->nameservers);
